/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "simplewindow.h"
#include "mainwindow.h"
#include "message.h"
#include "userlist.h"

#include <qtextview.h>
#include <qlineedit.h>
#include <qvbox.h>
#include <qsplitter.h>

#include <kstringhandler.h>

SimpleWindow::SimpleWindow( const QString &channel, IRC::Frontend *frontend, const char *name )
    : IRC::Window( channel, frontend, name )
{
    m_box = new QVBox( MainWindow::self().centralWidget() );

    QSplitter *splitter = new QSplitter( m_box );
    splitter->setOrientation( Horizontal );

    m_view = new QTextView( splitter );
    m_view->setTextFormat( RichText );

    m_userList = new UserList( splitter );
    connect( this, SIGNAL( userUpdated( const IRC::Person &, const IRC::Person & ) ),
             m_userList, SLOT( updateUser( const IRC::Person &, const IRC::Person & ) ) );

    connect( this, SIGNAL( userUpdated( const IRC::Person &, const IRC::Person & ) ),
             m_userList, SLOT( show() ) );

    connect( this, SIGNAL( userRemoved( const IRC::Person & ) ),
             m_userList, SLOT( removeUser( const IRC::Person & ) ) );
    connect( m_userList, SIGNAL( highlighted( const QString & ) ),
             this, SLOT( emitStatusMessageForUserHighlight( const QString & ) ) );

    m_input = new QLineEdit( m_box );
    connect( m_input, SIGNAL( returnPressed() ),
             this, SLOT( processInput() ) );

    m_box->setCaption( channel );

    m_box->setFocusProxy( m_input );

    QValueList<int> sizes;
    sizes << 80 << 20;
    splitter->setSizes( sizes );

    m_userList->hide();

    MainWindow::self().addTabView( m_box );
}

SimpleWindow::~SimpleWindow()
{
    delete m_box;
}

QString SimpleWindow::title() const
{
    return m_box->caption();
}

void SimpleWindow::setTitle( const QString &text )
{
    m_box->setCaption( channel() + ": " + text );
}

void SimpleWindow::printMessage( const QString &text )
{
    m_view->append( KStringHandler::tagURLs( QStyleSheet::escape( text ) ) );
}

void SimpleWindow::processInput()
{
    QString text = m_input->text();
    processInput( text );
    m_input->clear();
}

void SimpleWindow::emitStatusMessageForUserHighlight( const QString &nick )
{
    const IRC::PersonList &userList = users();
    IRC::PersonList::ConstIterator person = userList.find( IRC::Person( nick ) );
    if ( person == userList.end() )
        return;

    emit statusMessage( ( *person ).toMaskString() );
}

#include "simplewindow.moc"

/* vim: et sw=4 ts=4
 */
